#!/usr/bin/env node
'use strict';

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _batchPromises = require('batch-promises');

var _batchPromises2 = _interopRequireDefault(_batchPromises);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var babel = require("babel-core");


var argv = require('yargs').usage('Usage: $0 --').option('copy-files', {
    alias: 'c',
    describe: 'copy non-babel files'
}).option('src-dir', {
    alias: 's',
    describe: 'src folder'
}).option('out-dir', {
    alias: 'o',
    describe: 'output folder'
}).option('skip-initial-build', {
    alias: 'i',
    describe: 'skip inital build'
}).option('watch', {
    alias: 'w',
    describe: 'watch source files a n rebuild as needed'
}).help().alias('help', 'h').example('$0', 'compile').argv;

var srcRegExp = new RegExp('^./' + argv['src-dir'] + '/');
var babelRegExp = new RegExp('^.jsx?$');

var compileFile = function compileFile(srcF) {
    return new Promise(function (resolve, reject) {

        var destF = srcF.replace(srcRegExp, './' + argv['out-dir'] + '/').replace(/\.jsx$/, '.js');
        var logStr = srcF + ' -> ' + destF;

        //let's see if we have in dest and it is newer that in src

        var srcStat = _fs2.default.existsSync(srcF) && _fs2.default.statSync(srcF);
        var destStat = _fs2.default.existsSync(destF) ? _fs2.default.statSync(destF) : { mtime: new Date(-1) };

        if (srcStat.mtime.getTime() < destStat.mtime.getTime()) {
            console.log(logStr + ' ...skip');
            return resolve({ srcF: srcF });
        }

        _mkdirp2.default.sync(_path2.default.dirname(destF));

        if (!/\.jsx?$/.test(srcF)) {
            if (argv['copy-files']) {
                console.log(logStr + ' ...copy');
                _fs2.default.writeFileSync(destF, _fs2.default.readFileSync(srcF));
                return resolve({ srcF: srcF });
            }
            console.log(logStr + ' ...skip');
            return resolve({ srcF: srcF });
        }

        babel.transformFile(srcF, {}, function (err, res) {
            if (err) {
                return reject({ srcF: srcF, err: err });
            }
            _fs2.default.writeFile(destF, res.code, function (err) {
                if (err) {
                    return reject({ srcF: srcF, err: err });
                }
                console.log(logStr + ' ...babel');
                resolve({ srcF: srcF });
            });
        });
    });
};

var compile = function compile(files) {
    (0, _batchPromises2.default)(20, files, function (f) {
        return compileFile(f);
    }).then(function (results) {}).catch(function (err) {
        console.log(err);
        process.exit(-1);
    });
};

var getFiles = function getFiles() {
    return _glob2.default.sync('./' + argv['src-dir'] + '/**/*', { nodir: true });
};

if (!argv["skip-initial-build"]) {
    compile(getFiles());
}

if (argv.watch) {
    console.log('start watching');

    _fs2.default.watch(argv['src-dir'], { recursive: true }, function (et, fn) {
        var files = [];
        console.log("detected:", et, fn);

        switch (et) {
            case "change":
                compile(['./' + argv['src-dir'] + '/' + fn]);
                break;
            default:
                // in all other cases do full rebuild for now
                compile(getFiles());
                break;
        }
    });
}